module Day5Lib
  ( Crate (..),
    Command (..),
    crate,
    crateOrNothing,
    crateRow,
    crateNumbers,
    command,
    transposeRows,
    --
    processInput1,
    processInput2,
  )
where

import Debug.Trace
import Text.Parsec

newtype Crate = Crate Char deriving (Show, Eq)

data Command = Command
  { cnt :: Int,
    from :: Int,
    to :: Int
  }
  deriving (Show, Eq)

crate :: Parsec String () Crate
crate = do
  char '['
  crateLabel <- oneOf ['A' .. 'Z']
  char ']'
  return $ Crate crateLabel

nothing = do
  string "   "
  return Nothing

crateOrNothing :: Parsec String () (Maybe Crate)
crateOrNothing = nothing <|> (crate >>= return . return)

crateRow :: Parsec String () [Maybe Crate]
crateRow = do
  c <- crateOrNothing `sepBy1` (char ' ')
  char '\n'
  return c

crateNumbers :: Parsec String () ()
crateNumbers = do
  char ' '
  oneOf ['1' .. '9'] `sepBy1` try (string "   ")
  char ' '
  char '\n'
  return ()

command :: Parsec String () Command
command = do
  string "move "
  countS <- many $ oneOf ['0' .. '9']
  char ' '
  string "from "
  from <- many $ oneOf ['0' .. '9']
  char ' '
  string "to "
  to <- many $ oneOf ['0' .. '9']
  return Command {cnt = read countS, from = read from - 1, to = read to - 1}

taskFile = do
  crateRows <- many (try crateRow)
  crateNumbers
  char '\n'
  commands <- sepBy command (char '\n')
  eof
  return (transposeRows crateRows, commands)

transposeRows :: [[Maybe Crate]] -> [[Crate]]
transposeRows = foldr applyRow (repeat [])

applyRow :: [Maybe Crate] -> [[Crate]] -> [[Crate]]
applyRow (Nothing : ow) (s : tacks) = s : (applyRow ow tacks)
applyRow ((Just c) : ow) (s : tacks) = (c : s) : (applyRow ow tacks)
applyRow [] stacks = []
applyRow _ [] = error "stacks error"

applyCommand1 :: [[Crate]] -> Command -> [[Crate]]
applyCommand1 crates cmd = last . take (cnt cmd + 1) . iterate applyThisMove $ crates
  where
    applyThisMove = applyMove 1 (from cmd) (to cmd)

applyCommand :: [[Crate]] -> Command -> [[Crate]]
applyCommand crates cmd = applyMove (cnt cmd) (from cmd) (to cmd) crates

applyMove :: Int -> Int -> Int -> [[Crate]] -> [[Crate]]
applyMove cnt from to crates =
  let (lf, f : rf) = splitAt from crates
      (lt, t : rt) = splitAt to (lf ++ (drop cnt f) : rf)
   in lt ++ ((take cnt f) ++ t) : rt

-- ####################

processInput1 :: String -> String
processInput1 input =
  let (crates, commands) = case parse taskFile "" input of
        Left e -> error (show e)
        Right p -> p
   in map ((\(Crate a) -> a) . head) $ foldl applyCommand1 crates commands

processInput2 :: String -> String
processInput2 input =
  let (crates, commands) = case parse taskFile "" input of
        Left e -> error (show e)
        Right p -> p
   in map ((\(Crate a) -> a) . head) $ foldl applyCommand crates commands
